/*
 * This file is part of firk's Common C Library
 *  io.h - signal-safe buffered file i/o
 *
 * (c) firk 2020; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef FCL_INCLUDED_IO_H
#define FCL_INCLUDED_IO_H

typedef struct _FCL_FILE {
  int     fd;
  int     err;
  uint    flags;
  uint    rsvd;
  uint8 * r_pos; /* where to read next char */
  uint8 * r_end; /* where to stop reading from buffer */
  uint8 * w_pos; /* where to write next char; non-NULL = buffering enabled */
  uint8 * w_end; /* where to write last char; this differs from r_end logic */
  uint8 * r_buf;
  uint8 * w_buf;
  size_t  r_size;
  size_t  w_size;
  char const volatile * allow_intr;
} FCL_FILE;
/*
   what should never be violated:
     !r_pos || r_pos>=r_buf && r_pos<=r_end
     !w_pos || w_pos>=w_buf && w_pos<=w_end && (flags & FCL_WRITE_BUF)
   in other words:
     non-null _pos should be always in range _buf ... _end
     last byte in write-buffer should always be free
     non-null w_pos should be only when writebuf enabled
 */
#define FCL_READ          1
#define FCL_WRITE         2
#define FCL_APPEND        4
#define FCL_NOSEEK        8
#define FCL_READ_AHEAD    0x1000
#define FCL_WRITE_LINEBUF 0x2000
#define FCL_WRITE_BUF     0x4000
#define FCL_FERROR        0x8000
 /* FCL_NOSEEK:
      no seeks allowed;
      no flushing of read-ahead allowed or needed before write */

#define FCL_ALLOW_INTR "\001"
#define FCL_IGNORE_INTR ""

#define FCL_DEFAULT_BUFSIZE 1024


#define AI char const volatile * allow_intr
int fcl_fread(FCL_FILE * fp, void * buf, size_t sz, size_t * rsz, AI);
int fcl_fread_all(FCL_FILE * fp, void * buf, size_t sz, size_t * rsz, AI);
int fcl_fflush(FCL_FILE * fp, AI);
int fcl_fgetc(FCL_FILE * fp, char * c, AI);
int fcl_fputc(FCL_FILE * fp, char c, AI);
int fcl_fgets(FCL_FILE * fp, char * dest, size_t destsz, size_t * cursz, AI);
int fcl_fputs(FCL_FILE * fp, char * st, size_t * cursz, AI);

/* NOTE: _w_end is a pointer to the last writable position of buffer;
         flushing should be triggered when writing to that position */
#ifdef FCL_INLINE
FCL_INLINE int fcl_putc(FCL_FILE * fp, char c, AI) {
  if(fp->w_pos && fp->w_pos<fp->w_end &&
     (c!='\n' || !(fp->flags & FCL_WRITE_LINEBUF))) {
    *(fp->w_pos++) = (uint8)c;
    return 1;
  } else return fcl_fputc(fp, c, allow_intr);
}

FCL_INLINE int fcl_putc_fast(FCL_FILE * fp, char c, AI) {
  if(fp->w_pos && fp->w_pos<fp->w_end) {
    *(fp->w_pos++) = (uint8)c;
    return 1;
  } else return fcl_fputc(fp, c, allow_intr);
}

FCL_INLINE int fcl_getc(FCL_FILE * fp, char * c, AI) {
  if(fp->r_pos && fp->r_pos<fp->r_end) {
    *c = *(fp->r_pos++);
    return 1;
  } else return fcl_fgetc(fp, c, allow_intr);
}
#else
#define fcl_putc(fp,c,ai) \
   ( \
     ( \
       (fp)->w_pos && (fp)->w_pos<(fp)->w_end && \
       (c!='\n' || !((fp)->flags & FCL_WRITE_LINEBUF)) \
     ) ? ( \
       (*((fp)->w_pos++) = (uint8)(c)),1 \
     ) : ( \
       fcl_fputc(fp,c,ai) \
     ) \
   )

#define fcl_putc_fast(fp,c,ai) \
   ( \
     ( \
       (fp)->w_pos && (fp)->w_pos<(fp)->w_end \
     ) ? ( \
       (*((fp)->w_pos++) = (uint8)(c)),1 \
     ) : ( \
       fcl_fputc(fp,c,ai) \
     ) \
   )

#define fcl_getc(fp,c,ai) \
   ( \
     ( \
       (fp)->r_pos && (fp)->r_pos<(fp)->r_end \
     ) ? ( \
       (*(c) = *((fp)->r_pos++)), 1 \
     ) : ( \
       fcl_fgetc(fp,c,ai) \
     ) \
   )
#endif

#undef AI

#endif

