/*
 * This file is part of firk's Common C Library
 *  qlog.h - quick logging
 *
 * (c) firk 2019-2022; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef FCL_INCLUDED_QLOG_H
#define FCL_INCLUDED_QLOG_H

/* this is mostly copypasted from farc archive/log_default.c ~2019-2020 */
/* !!! incompatible changes:
   %s is now usual string, %S is escaped string
   %c prints just a character like printf
   %C prints escaped character and does not skip \0
   %L -> %llu
   !!! improvements:
   now supported %{width}{mod}{type} format specifiers
   NOTE: .{precision} not supported for now
   */

/* purpose: simple and quick way to log critical events or just console app log
   not intended for use by daemons
     just exit(-1) on any problems
     weird characters and quotes may be escaped when needed
     not using heap
     not using stdio
     line-buffered
 */

/*
  escaped char/string means escaped \a \b \t \n \r \" \\ \177
    and all other symbols in 0..31 range as \ooo  (\000 used isntad of \0)
  allowed non-character formats:
    %d = signed integer
    %u = unsigned integer
    %x = unsigned integer as hex a-f
    %X = unsigned integer as hex A-F
    %o = unsigned integer as octal
  modifiers:
    hh = char, h = short, l = long, ll = llong, z = size_t
    HH = 8bit, H = 16bit, L = 32bit, LL = 64bit
  width:
    currently 0 treated as 1 but this may change
 */

extern void qlog_set_fd(int fd);
extern void qlog_set_times(int flag);
extern void qlog_set_prefix(char const * pre);
extern void qlog_set_errh(int flags);

#define QLOG_TIMES_NO           0
#define QLOG_TIMES_SEC          1
#define QLOG_TIMES_USEC         2

#define QLOG_ERRH_WAITREC 1 /* wait for errors that could be recovered */
#define QLOG_ERRH_IGNORE  2 /* ignore write errors */
#define QLOG_ERRH_IGNFMT  4 /* ignore badfmt errors */

#define QLOG_CHECK_ENV          64 /* for times and errh */

extern void qlog_pending_newline(int flag);
extern void qlog_begin_line(void);
extern void qlog_end_line(void);
extern void qlog_flush(void);

extern void qlog(char const * fmt, ...);
extern void qlog_q(char const * fmt, ...);
extern void qlog_printf(char const * fmt, ...);
#ifdef FCL_WITH_VA_LIST /* this is because stdarg.h needed */
extern void qlog_vprintf(char const * fmt, va_list arg);
#endif
extern void qlog_putc(char c);
extern void qlog_putce(char c);
extern void qlog_puts(char const * s);
extern void qlog_putse(char const * s);
extern void qlog_putb(char const * s, fcl_size len);
extern void qlog_putbe(char const * s, fcl_size len);

#define FCL_QLOG_BUFSIZE 8192

#endif

