/*
 * This file is part of firk's Common C Library
 *  strparse.h - string parsing functions
 *
 * (c) firk 2015, 2019, 2020, 2021, 2023; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef INCLUDED_FCL_STRPARSE_H
#define INCLUDED_FCL_STRPARSE_H

#ifdef FCL_SIMPLE_NAMES
#define skip_spaces str_skip_spaces
#define skip_prefix str_skip_prefix
#define skip_untilsep str_skip_untilsep
#endif

/*** str_skip_spaces()
 * if st==NULL, return NULL
 * skip all leading ' ' and '\t' characters and return new pointer
 */
extern char const * str_skip_spaces(char const * st);

/*** str_skip_prefix()
 * if st==NULL, return NULL
 * if prefix==NULL, UB
 * if st prefixed with prefix, return its tail after prefix
 * else return NULL
 */
extern char const * str_skip_prefix(char const * st, char const * prefix);

/*** str_skip_untilsep()
 * if st==NULL, return NULL
 * if prefix==NULL, UB
 * if st prefixed with prefix, and next byte is \0 or contained in sep
 *   return its tail after prefix
 * else return NULL
 */
extern char const * str_skip_untilsep(char const * st, char const * prefix,
                                      char const * sep);

/*** str_to_args()
 * parse string as command-line arguments
 * reuse its bytes to store results
 * return parsed arguments in argv[]
 */
extern char str_to_args(char * st, char * * argv, size_t * argc, size_t max,
                        uint flags);

/*** chr_to_value()
 * for '0'..'9', return 0..9
 * for 'A'..'Z', return 10..35
 * for 'a'..'z', return 10..35
 * for others, return 255
 */
extern unsigned char chr_to_value(char c);

/*** str_get_bool()
 * for "0", "no", "false", "n", "off" return 0
 * for "1", "yes", "true", "y", "on"  return 1
 * for others, return -1
 * comparsion is case-insensitive
 */
extern int str_get_bool(char const * st);

/*** str_to_TYPE()
 * if st!=NULL and is a valid number (just number, no spaces and etc) that
 *  fits to TYPE, store it into *res and return 0
 * return -1 in any other case, and don't modify *res;
 * empty string considered as invalid
 * all numbers parsed as decimal
 */
/*** str_hex_to_TYPE()
 * same as str_to_TYPE, but all number parsed as hexadecimal
 * "0x" prefix is not accepted and considered as invalid string
 */
/*** str_base_to_TYPE()
 * same as str_to_TYPE, but all number parsed as specified base
 */
/*** strp_to_TYPE()
 * parse *st until invalid character occured
 * st==NULL or *st==NULL => invalid input
 * store resulting position to *st
 * at least one numeric charracter required
 * strp_to_uchar "34p23" will parse 34, return 0 and store new pos at "p23"
 * strp_to_uchar "3423" will overflow at "23", return -1 and not modify pos/res
 * all numbers parsed as decimal
 */
#define DECL_PARSEFUNC(name, utype, stype) \
  extern int str_to_u##name(char const * st, utype * res); \
  extern int str_to_s##name(char const * st, stype * res); \
  extern int str_hex_to_u##name(char const * st, utype * res); \
  extern int str_base_to_u##name(char const * st, utype * res, uchar b); \
  extern int str_base_to_s##name(char const * st, stype * res, uchar b); \
  extern int strp_to_u##name(char const * * st, utype * res); \
  extern int strp_to_s##name(char const * * st, stype * res); \
  extern int strp_base_to_u##name(char const * * st, utype * res, uchar b); \
  extern int strp_base_to_s##name(char const * * st, stype * res, uchar b);

DECL_PARSEFUNC(char, unsigned char, signed char)
DECL_PARSEFUNC(short, unsigned short, signed short)
DECL_PARSEFUNC(int, unsigned int, signed int)
DECL_PARSEFUNC(long, unsigned long, signed long)
#ifdef HAVE_LONGLONG
DECL_PARSEFUNC(llong, unsigned long long, signed long long)
#endif
#ifdef HAVE_INT8
DECL_PARSEFUNC(int8, uint8, int8)
#endif
#ifdef HAVE_INT16
DECL_PARSEFUNC(int16, uint16, int16)
#endif
#ifdef HAVE_INT32
DECL_PARSEFUNC(int32, uint32, int32)
#endif
#ifdef HAVE_INT64
DECL_PARSEFUNC(int64, uint64, int64)
#endif
#ifdef HAVE_INT128
DECL_PARSEFUNC(int128, uint128, int128)
#endif
DECL_PARSEFUNC(size, fcl_size, fcl_ssize)
DECL_PARSEFUNC(xsize, fcl_size, fcl_ssize)

#undef DECL_PARSEFUNC

#define FP_NULL_INPUT         -1
#define FP_SIGN_DISALLOWED    -2
#define FP_MISSING_DIGITS     -3
#define FP_MISSING_BEFORE_DOT -4
#define FP_MISSING_BEFORE_EXP -5
#define FP_MISSING_EXP        -6
#define FP_OVERFLOW_EXP       -7
#define FP_EXTRA_SYMBOLS      -8
#define FP_OVERFLOW           -9

#define DECL_PARSEFUNC(name, type) \
  extern int str_to_##name(char const * st, type * res, uint flags); \
  extern int strp_to_##name(char const * * st, type * res, uint flags);
DECL_PARSEFUNC(float, float)
DECL_PARSEFUNC(double, double)
DECL_PARSEFUNC(ldouble, long double)
#undef DECL_PARSEFUNC

extern int str_to_ip4_addr(char const * st, uint32 * ip);
extern int str_to_ip4_addr_port(char const * st, uint32 * ip, uint16 * port);
extern int str_to_eth_addr(char const * st, uint8 * mac);
extern int str_to_uid(char const * st, uint32 * uid);
extern int str_to_gid(char const * st, uint32 * gid);
extern int str_to_uid_gid(char const * st, uint32 * uid, uint32 * gid);
extern int strp_to_ip4_addr(char const * * st, uint32 * ip);
extern int strp_to_ip4_addr_port(char const ** st, uint32 * ip, uint16 * port);
extern int strp_to_eth_addr(char const * * st, uint8 * mac);

#endif

